/* 
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 *
 */

package org.apache.muse.core.platform.osgi;

import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceEvent;
import org.osgi.framework.ServiceReference;
import org.osgi.framework.ServiceListener;
import org.osgi.service.log.LogService;
import org.osgi.service.log.LogReaderService;

import org.apache.muse.core.platform.osgi.routing.OSGiResourceRouter;
import org.apache.muse.core.platform.osgi.util.OSGiReflectUtilHelper;

/**
 * 
 * Activator is the <code>BundleActivator</code> implementation for the Muse
 * Core Bundle. The activator manages the OSGi Service interface to the
 * appropriate IsolationLayer implementation through an instance of the
 * <code>org.apache.muse.core.platform.osgi.ResourceManagementProvider</code>
 * service.
 * 
 * @see org.apache.muse.core.platform.osgi.ResourceManagementProvider
 * @author Joel Hawkins (joelh)
 * 
 */

public class Activator implements BundleActivator
{
    
    private static final String MANAGEMENT_ENVIRONMENT_FILTER = "(objectclass=org.apache.muse.core.platform.osgi.ResourceManagementProvider)";
    
    private static final String LOG_SERVICE_FILTER = "(objectclass=org.osgi.service.log.LogService)";
    
    private static final String LOG_READER_SERVICE_FILTER = "(objectclass=org.osgi.service.log.LogReaderService)";
    
    private ServiceReference managementServiceRef;
    
    private ServiceReference logServiceReference = null;
    
    private ServiceReference logReaderServiceReference = null;
    
    private static Activator activator;
    
    private BundleContext context;
    
    private ResourceManagementProvider provider;
    
    static private class ManagementServiceListener implements ServiceListener
    {
        
        public void serviceChanged(ServiceEvent event)
        {
            ServiceReference ref = event.getServiceReference();
            switch (event.getType())
            {
                case ServiceEvent.REGISTERED:
                    if (activator.managementServiceRef == null)
                    {
                        activator.managementServiceRef = event.getServiceReference();
                        activator.provider = (ResourceManagementProvider) activator.context.getService(activator.managementServiceRef);
                        OSGiResourceRouter.getDefault().attachManagementProvider(activator.provider);
                    }
                    break;
                case ServiceEvent.MODIFIED:
                    break;
                case ServiceEvent.UNREGISTERING:
                    ref = event.getServiceReference();
                    if (ref == activator.managementServiceRef)
                    {
                        activator.managementServiceRef = null;
                        OSGiResourceRouter.getDefault().releaseManagementProvider();
                    }
                    break;
            }
        }
    }
    
    private ManagementServiceListener managementListener = new ManagementServiceListener();
    
    static private class LogServiceListener implements ServiceListener
    {
        private boolean enabled = false;
        
        public void serviceChanged(ServiceEvent event)
        {
            ServiceReference ref = event.getServiceReference();
            switch (event.getType())
            {
                case ServiceEvent.REGISTERED:
                    if (!enabled)
                    {
                        try
                        {
                            activator.logServiceReference = activator.context.getServiceReference("org.osgi.service.log.LogService");
                            
                            if (activator.logServiceReference != null)
                            {
                                LogService logService = (LogService) activator.context.getService(activator.logServiceReference);
                                OSGiLogHandler.setLogService(logService);
                                enabled = true;
                            }
                            activator.logReaderServiceReference = activator.context.getServiceReference("org.osgi.service.log.LogReaderService");
                            if (activator.logReaderServiceReference != null)
                            {
                                LogReaderService logReaderService = (LogReaderService) activator.context.getService(activator.logReaderServiceReference);
                                OSGiLogHandler.setLogReaderService(logReaderService);
                            }
                        }
                        catch (Exception e)
                        {
                            e.printStackTrace();
                        }
                    }
                    break;
                case ServiceEvent.MODIFIED:
                    break;
                case ServiceEvent.UNREGISTERING:
                    if (ref == activator.logServiceReference)
                        OSGiLogHandler.setLogService(null);
                    
                    if (ref == activator.logReaderServiceReference)
                        OSGiLogHandler.setLogReaderService(null);
                    
                    enabled = false;
                    break;
            }
            
        }
        
    }
    
    private LogServiceListener logListener = new LogServiceListener();
    
    /**
     * starts the Bundle. The <code>start</code> method attempts to resolve an
     * initial refrence to the <code>ResourceManagementProvider</code> service
     * and attach that service to the <code>OSGiResourceRouter</code>
     * instance. The method also intializes the OSGiReflectUtilHelper instance.
     * 
     * @see org.apache.muse.core.platform.osgi.routing.OSGiResourceRouter
     * @see org.apache.muse.core.platform.osgi.util.OSGiReflectUtilHelper
     * @see org.osgi.framework.BundleActivator#start(org.osgi.framework.BundleContext)
     */
    public void start(BundleContext context) throws Exception
    {
        activator = this;
        this.context = context;
        try
        {
            OSGiResourceRouter.getDefault().initialize(context);
        }
        catch (Throwable t)
        {
            t.printStackTrace();
        }
        
        if ((managementServiceRef = context.getServiceReference("org.apache.muse.core.platform.osgi.ResourceManagementProvider")) != null)
        {
            activator.provider = (ResourceManagementProvider) activator.context.getService(activator.managementServiceRef);
            OSGiResourceRouter.getDefault().attachManagementProvider(activator.provider);
        }
        
        OSGiReflectUtilHelper.getDefault().initialize(context);
        
        context.addServiceListener(managementListener, MANAGEMENT_ENVIRONMENT_FILTER);
        context.addServiceListener(logListener, LOG_SERVICE_FILTER);
        context.addServiceListener(logListener, LOG_READER_SERVICE_FILTER);
    }
    
    /**
     * stops the Bundle and removes any applied listeners
     *
     * @see org.osgi.framework.BundleActivator#stop(org.osgi.framework.BundleContext)
     */
    public void stop(BundleContext context) throws Exception
    {
        context.removeBundleListener(OSGiResourceRouter.getDefault());
        context.removeServiceListener(managementListener);
        context.removeServiceListener(logListener);
    }
    
}
