/* 
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 *
 */

package java.util.logging;

import java.util.*;

/**
 * 
 * @author Barry Atkins
 * 
 */

public class Logger
{
    private static final Map _LOGGERS = new HashMap();
    
    private List _handlers = new LinkedList();
    
    private Level _level = Level.ALL;
    
    private String _name = null;
    
    private Logger(String name)
    {
        _name = name;
    }
    
    public void addHandler(Handler handler)
    {
        handler.setLevel(getLevel());
        _handlers.add(handler);
    }
    
    public void config(String message)
    {
        log(Level.CONFIG, message);
    }
    
    public void fine(String message)
    {
        log(Level.FINE, message);
    }
    
    public void finer(String message)
    {
        log(Level.FINER, message);
    }
    
    public void finest(String message)
    {
        log(Level.FINEST, message);
    }
    
    public Handler[] getHandlers()
    {
        Handler[] array = new Handler[_handlers.size()];
        return (Handler[])_handlers.toArray(array);
    }
    
    public Level getLevel()
    {
        return _level;
    }
    
    public static Logger getLogger(String name)
    {
        if (!_LOGGERS.containsKey(name))
            _LOGGERS.put(name, new Logger(name));
        
        return (Logger)_LOGGERS.get(name);
    }
        
    public String getName()
    {
        return _name;
    }
    
    public void info(String message)
    {
        log(Level.INFO, message);
    }
    
    public boolean isLoggable(LogRecord record)
    {
        return getLevel().intValue() >= record.getLevel().intValue();
    }
    
    public void log(Level level, String message)
    {
        LogRecord record = new LogRecord(level, message);
        
        if (!isLoggable(record))
            return;
        
        Iterator i = _handlers.iterator();
        
        while (i.hasNext())
        {
            Handler next = (Handler)i.next();
            next.publish(record);
        }
    }
    
    public void removeHandler(Handler handler)
    {
        _handlers.remove(handler);
    }
    
    public void setLevel(Level level)
    {
        _level = level;
    }
    
    public void severe(String message)
    {
        log(Level.SEVERE, message);
    }
    
    public String toString()
    {
        return getName();
    }
    
    public void warning(String message)
    {
        log(Level.WARNING, message);
    }
}
