/* 
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 *
 */

package org.apache.muse.ws.notification.impl;

import javax.xml.namespace.QName;
import javax.xml.transform.TransformerException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import org.apache.muse.util.messages.Messages;
import org.apache.muse.util.messages.MessagesFactory;
import org.apache.muse.util.xml.XPathUtils;
import org.apache.muse.util.xml.XmlUtils;
import org.apache.muse.ws.notification.Filter;
import org.apache.muse.ws.notification.NotificationMessage;
import org.apache.muse.ws.notification.WsnConstants;
import org.apache.muse.ws.notification.faults.*;
import org.apache.muse.ws.notification.topics.Topic;
import org.apache.muse.ws.notification.topics.WstConstants;
import org.apache.muse.ws.notification.topics.impl.ConcretePathExpression;

/**
 * 
 * TopicFilter is the implementation of topic-based filters for WS-N subscriptions. 
 * All messages that have a wsnt:Topic field that matches the filter's topic 
 * will be sent to subscribers. Only simple and concrete topic expressions are 
 * supported by this filter.
 *
 * @author Dan Jemiolo (danj)
 *
 */

public class TopicFilter implements Filter
{
    //
    // Used to look up all exception messages
    //
    private static Messages _MESSAGES = MessagesFactory.get(TopicFilter.class);
    
    private QName _topicName = null;
    
    private Topic _topic = null;
    
    public TopicFilter(QName topicName) 
        throws TopicExpressionDialectUnknownFault, 
               InvalidTopicExpressionFault
    {
        this(topicName, WstConstants.CONCRETE_TOPIC_URI);
    }
    
    public TopicFilter(QName topicName, String dialect) 
        throws TopicExpressionDialectUnknownFault, 
               InvalidTopicExpressionFault
    {
        //
        // we only support simple and concrete topic expressions
        //
        if (!dialect.equals(WstConstants.CONCRETE_TOPIC_URI))
        {
            Object[] filler = { dialect, WstConstants.CONCRETE_TOPIC_URI };
            throw new TopicExpressionDialectUnknownFault(_MESSAGES.get("InvalidDialect", filler));
        }
        
        ConcretePathExpression.validateTopicPath(topicName);
        
        _topicName = topicName;
    }
    
    public TopicFilter(Topic topic) 
        throws TopicExpressionDialectUnknownFault, 
               InvalidTopicExpressionFault
    {
        this(topic, WstConstants.CONCRETE_TOPIC_URI);
    }
    
    public TopicFilter(Topic topic, String dialect) 
        throws TopicExpressionDialectUnknownFault, 
               InvalidTopicExpressionFault
    {
        this(topic.getConcretePath(), dialect);
    }
    
    public boolean accepts(NotificationMessage message)
    {
        QName messageTopicName = message.getTopic();
        
        //
        // no topic name? not from our topic
        //
        if (messageTopicName == null)
            return false;
        
        //
        // different topic name? not from our topic
        //
        if (!messageTopicName.equals(_topicName))
            return false;
        
        //
        // if we haven't supplied the Topic data structure, we 
        // can't compare message types/patterns - must be a match
        //
        if (_topic == null)
            return true;
        
        String pattern = _topic.getMessagePattern();
        
        //
        // no pattern to compare? it's a match
        //
        if (pattern == null)
            return true;
        
        //
        // we only support XPath patterns in the default impl
        //
        try
        {
            return XPathUtils.isMatch(message.toXML(), pattern);
        }
        
        catch (TransformerException error)
        {
            throw new RuntimeException(error.getMessage(), error);
        }
    }
    
    public QName getTopic()
    {
        return _topicName;
    }
    
    public void setTopic(QName topicName)
    {
        _topicName = topicName;
    }
    
    public void setTopic(Topic topic)
    {
        _topic = topic;
    }
    
    public String toString()
    {
        return XmlUtils.toString(toXML(), false);
    }

    public Element toXML()
    {
        return toXML(XmlUtils.EMPTY_DOC);
    }

    public Element toXML(Document doc)
    {
        Element filter = XmlUtils.createElement(doc, WsnConstants.FILTER_QNAME);
        
        Element topic = XmlUtils.createElement(doc, WsnConstants.TOPIC_EXPRESSION_QNAME, getTopic());
        topic.setAttribute(WsnConstants.DIALECT, WstConstants.CONCRETE_TOPIC_URI);
        filter.appendChild(topic);
        
        return filter;
    }
}
